@@ -41,8 +41,12 @@
 #include <QMenu>
 #include <QMenuBar>
 #include <QStatusBar>
+#include <QDBusReply>
+#include <QDBusInterface>
+#include <QDBusUnixFileDescriptor>
 
-//#include <KNotificationRestrictions> // kf5 FIXME
+#include <KAboutData>
+#include <KNotificationRestrictions>
 #include <KSqueezedTextLabel>
 #include <KToggleFullScreenAction>
 #include <KToolBar>
@@ -61,9 +65,6 @@
 
 #include <Solid/Device>
 #include <Solid/OpticalDisc>
-#if 0 // kf5 FIXME port to new Solid::Power API
-#include <solid/powermanagement.h>
-#endif
 
 #include "actions.h"
 #include "discSelectionDialog.h"
@@ -90,7 +91,7 @@ MainWindow::MainWindow()
     , m_mainView( 0 )
     , m_audioView( 0 )
     , m_loadView( new LoadView(this) )
-    , m_currentWidget( new QWidget(this) )
+    , m_currentWidget( nullptr )
     , m_leftDock( 0 )
     , m_positionSlider( 0 )
     , m_volumeSlider( 0 )
@@ -124,8 +125,6 @@ MainWindow::MainWindow()
     m_mainView->addWidget(videoWindow());
     m_mainView->setCurrentWidget(m_loadView);
 
-    m_currentWidget = m_loadView;
-
     setCentralWidget( m_mainView );
 
     setFocusProxy( videoWindow() ); // essential! See VideoWindow::event(), QEvent::FocusOut
@@ -212,6 +211,8 @@ MainWindow::init()
     connect( engine(), SIGNAL(audioChannelsChanged(QList<QAction*>)), this, SLOT(audioChannelsChanged(QList<QAction*>)) );
     connect( engine(), SIGNAL(mutedChanged(bool)), this, SLOT(mutedChanged(bool)) );
 
+    connect( engine(), &VideoWindow::finished, this, &MainWindow::toggleLoadView );
+
     if( !engine()->init() ) {
         KMessageBox::error( this, i18n(
                                 "<qt>Phonon could not be successfully initialized. Dragon Player will now exit.</qt>") );
@@ -433,17 +434,20 @@ void
 MainWindow::toggleLoadView()
 {
     if( m_mainView->currentWidget() == m_loadView ) {
-        if( m_mainView->indexOf(m_currentWidget) == -1 ) {
-            m_mainView->addWidget(m_currentWidget);
+        if( engine()->state() != Phonon::StoppedState ) {
+            if( m_mainView->indexOf(m_currentWidget) == -1 ) {
+                m_mainView->addWidget(m_currentWidget);
+            }
+            m_mainView->setCurrentWidget(m_currentWidget);
         }
-        m_mainView->setCurrentWidget(m_currentWidget);
         engine()->isPreview(false);
     } else if( m_currentWidget != m_audioView ) {
-        qDebug() << "setting Thumbnail for video Widget";
-        m_mainView->setCurrentWidget(m_loadView);
-        m_mainView->removeWidget(m_currentWidget);
-        engine()->isPreview(true);
-        m_loadView->setThumbnail(m_currentWidget);
+        m_mainView->setCurrentWidget( m_loadView );
+        if( engine()->state() != Phonon::StoppedState ) {
+            m_mainView->removeWidget(m_currentWidget);
+            engine()->isPreview(true);
+            m_loadView->setThumbnail(m_currentWidget);
+        }
     } else {
         m_mainView->setCurrentWidget(m_loadView);
     }
@@ -493,7 +497,7 @@ MainWindow::mutedChanged( bool mute )
 void MainWindow::stop()
 {
     engine()->stop();
-    m_mainView->setCurrentWidget(m_loadView);
+    toggleLoadView();
 }
 
 void
@@ -568,22 +572,14 @@ MainWindow::load( const QUrl &url )
         job->deleteLater();
     }
 
-    if( m_mainView->indexOf(engine()) == -1 )
-        toggleLoadView();
-
     //let xine handle invalid, etc, QUrlS
     //TODO it handles non-existing files with bad error message
     if (!ret)
         ret = engine()->load( url );
 
     if( ret ) {
-        if( TheStream::hasVideo() ) {
-            m_currentWidget = engine();
-        } else {
-            m_currentWidget = m_audioView;
-            resize(m_currentWidget->minimumSize());
-        }
-        m_mainView->setCurrentWidget(m_currentWidget);
+        m_currentWidget = nullptr;
+        m_loadView->setThumbnail(nullptr);
     }
     return ret;
 }
@@ -601,14 +597,9 @@ MainWindow::play()
             toggleLoadView();
         break;
     case Phonon::StoppedState:
-        if( TheStream::hasVideo() ) {
-            m_currentWidget = engine();
-        } else {
-            m_currentWidget = m_audioView;
-            resize(m_currentWidget->minimumSize());
-        }
         engine()->play();
-        m_mainView->setCurrentWidget(m_currentWidget);
+        m_currentWidget = nullptr;
+        m_loadView->setThumbnail(nullptr);
         break;
     default:
         break;
@@ -689,6 +680,8 @@ MainWindow::playDisc()
         }
     } else {
         engine()->playDvd();
+        m_loadView->setThumbnail(nullptr);
+        toggleLoadView();
         qDebug() << "no disc in drive or Solid isn't working";
     }
 }
@@ -815,36 +808,49 @@ MainWindow::keyPressEvent( QKeyEvent *e )
 void
 MainWindow::inhibitPowerSave()
 {
-#if 0 // kf5 FIXME port to new Solid::Power API
-    if (m_stopSleepCookie == -1)
-        m_stopSleepCookie = Solid::PowerManagement::beginSuppressingSleep(QLatin1String( "watching a film" ));
-    if (m_stopScreenPowerMgmtCookie == -1 && TheStream::hasVideo())
-        m_stopScreenPowerMgmtCookie = Solid::PowerManagement::beginSuppressingScreenPowerManagement(QLatin1String( "watching a film" ));
+    if (m_stopSleepCookie == -1) {
+        QDBusInterface iface(QStringLiteral("org.freedesktop.login1"),
+                             QStringLiteral("/org/freedesktop/login1"),
+                             QStringLiteral("org.freedesktop.login1.Manager"),
+                             QDBusConnection::systemBus());
+        if (iface.isValid()) {
+            QDBusReply<QDBusUnixFileDescriptor> reply;
+            if (TheStream::hasVideo()) {
+                reply = iface.call(QStringLiteral("Inhibit"),
+                                   QStringLiteral("sleep:idle"),
+                                   KAboutData::applicationData().componentName(),
+                                   QStringLiteral("playing a video"),
+                                   QStringLiteral("block"));
+            } else {
+                reply = iface.call(QStringLiteral("Inhibit"),
+                                   QStringLiteral("sleep"),
+                                   KAboutData::applicationData().componentName(),
+                                   QStringLiteral("playing an audio"),
+                                   QStringLiteral("block"));
+            }
+            if (reply.isValid()) {
+                m_stopSleepCookie = reply.value().fileDescriptor();
+            }
+        }
+    }
+    // TODO: inhibit screen sleep. No viable API found.
+    // https://git.reviewboard.kde.org/r/129651
     if (!m_stopScreenSaver && TheStream::hasVideo())
         m_stopScreenSaver = new KNotificationRestrictions(KNotificationRestrictions::ScreenSaver);
-#endif
 }
 
 void
 MainWindow::releasePowerSave()
 {
-#if 0 // kf5 FIXME port to new Solid::Power API
     //stop supressing sleep
     if (m_stopSleepCookie != -1) {
-        Solid::PowerManagement::stopSuppressingSleep(m_stopSleepCookie);
+        ::close(m_stopSleepCookie);
         m_stopSleepCookie = -1;
     }
 
-    //stop supressing screen power management
-    if (m_stopScreenPowerMgmtCookie != -1) {
-        Solid::PowerManagement::stopSuppressingScreenPowerManagement(m_stopScreenPowerMgmtCookie);
-        m_stopScreenPowerMgmtCookie = -1;
-    }
-
     //stop disabling screensaver
     delete m_stopScreenSaver; // It is always 0, I have been careful.
     m_stopScreenSaver = 0;
-#endif
 }
 
 QMenu*
